package edu.uiuc.ncsa.security.oauth_1_0a.client;

import edu.uiuc.ncsa.security.core.exceptions.GeneralException;
import edu.uiuc.ncsa.security.delegation.client.request.CallbackRequest;
import edu.uiuc.ncsa.security.delegation.client.request.CallbackResponse;
import edu.uiuc.ncsa.security.delegation.client.server.CallbackServer;
import edu.uiuc.ncsa.security.delegation.token.AuthorizationGrant;
import edu.uiuc.ncsa.security.delegation.token.TokenForge;
import net.oauth.OAuth;

import javax.servlet.ServletRequest;
import java.net.URI;

import static net.oauth.OAuth.OAUTH_TOKEN;
import static net.oauth.OAuth.OAUTH_VERIFIER;

/**
 * <p>Created by Jeff Gaynor<br>
 * on May 12, 2011 at  1:14:51 PM
 */
public class CallbackServerImpl extends CallbackServer {
    public CallbackServerImpl(TokenForge tokenForge
            , URI address) {
        super(address);
        this.tokenForge = tokenForge;
    }

    TokenForge tokenForge;

    @Override
    public CallbackResponse processCallback(CallbackRequest callbackRequest) {
        CallbackResponse cResp = new CallbackResponse();
        ServletRequest servletRequest = callbackRequest.getServletRequest();
        String token = servletRequest.getParameter(OAUTH_TOKEN);
        if (token == null || token.length() == 0) {
            throw new GeneralException("Error: No token found");
        }
        String tc = OAuth.decodePercent(token);
        String verifier = servletRequest.getParameter(OAUTH_VERIFIER);
        if (verifier == null || verifier.length() == 0) {
            throw new GeneralException("Error: No verifier found");
        }
        String v = OAuth.decodePercent(verifier);
        AuthorizationGrant ag = tokenForge.getAuthorizationGrant(tc);
        cResp.setAuthorizationGrant(ag);
        cResp.setVerifier(tokenForge.getVerifier(v));
        return cResp;
    }

}
