package edu.uiuc.ncsa.security.oauth_1_0a.client;

import edu.uiuc.ncsa.security.delegation.client.DelegationService;
import edu.uiuc.ncsa.security.delegation.client.request.*;
import edu.uiuc.ncsa.security.delegation.client.server.AuthorizationServer;
import edu.uiuc.ncsa.security.delegation.client.server.ResourceServer;
import edu.uiuc.ncsa.security.oauth_1_0a.AuthorizationGrantImpl;

import java.net.URI;

import static net.oauth.OAuth.OAUTH_TOKEN;

/**
 * An OAuth 1.0a implementation of the delegation service.
 * <p>Created by Jeff Gaynor<br>
 * on Apr 15, 2011 at  2:38:40 PM
 */
public class DelegationServiceImpl extends DelegationService {
    public DelegationServiceImpl(AuthorizationServer authorizationGrantServer,
                                 AuthorizationServer authorizationServer,
                                 ResourceServer resourceServer) {
        super(authorizationGrantServer, authorizationServer, resourceServer);
    }

    @Override
    public DelegatedAssetResponse processAssetRequest(DelegatedAssetRequest delegationAssetRequest) {
        // First step is to get the access token
        AccessTokenRequest atr = new AccessTokenRequest();
        atr.setAuthorizationGrant(delegationAssetRequest.getAuthorizationGrant());
        atr.setClient(delegationAssetRequest.getClient());
        atr.setVerifier(delegationAssetRequest.getVerifier());
        atr.setParameters(delegationAssetRequest.getParameters());
        AccessTokenResponse atResp = (AccessTokenResponse) getAuthorizationServer().process(atr);

        ProtectedAssetRequest paReq = new ProtectedAssetRequest();
        paReq.setClient(delegationAssetRequest.getClient());
        paReq.setAccessToken(atResp.getAccessToken());
        paReq.setParameters(delegationAssetRequest.getAssetParameters());
        ProtectedAssetResponse paResp = (ProtectedAssetResponse) getResourceServer().process(paReq);
        DelegatedAssetResponse dap = new DelegatedAssetResponse(paResp.getProtectedAsset());
        dap.setAdditionalInformation(paResp.getAdditionalInformation());
        return dap;
    }

    @Override
    public DelegationResponse processDelegationRequest(DelegationRequest delegationRequest) {
        AuthorizationGrantRequest agReq = new AuthorizationGrantRequest();
        agReq.setClient(delegationRequest.getClient());
        // Annoyingly, OAuth 1.0a library doesn't pick up on the callback. This ensures it gets to the right place,
        // since it always adds in all the parameters it finds in the map plus it removes duplicates.
        agReq.setParameters(delegationRequest.getParameters());
        AuthorizationGrantResponse agResp = (AuthorizationGrantResponse) getAuthorizationGrantServer().process(agReq);
        DelegationResponse delResp = new DelegationResponse(agResp.getAuthorizationGrant());
        AuthorizationGrantImpl tc = (AuthorizationGrantImpl) agResp.getAuthorizationGrant();
        // Create the basic response uri. Applications are free to add on to this.
        String urlString = delegationRequest.getBaseUri() +
                "?" + OAUTH_TOKEN + "=" + tc.getURIToken();
        URI redirectURI = URI.create(urlString);
        delResp.setRedirectUri(redirectURI);
        return delResp;
    }

}
