package edu.uiuc.ncsa.security.oauth_1_0a.server;

import edu.uiuc.ncsa.security.core.exceptions.GeneralException;
import edu.uiuc.ncsa.security.core.exceptions.NotImplementedException;
import edu.uiuc.ncsa.security.delegation.server.issuers.AuthorizationGrantIssuer;
import edu.uiuc.ncsa.security.delegation.server.request.AGRequest;
import edu.uiuc.ncsa.security.delegation.server.request.AGResponse;
import edu.uiuc.ncsa.security.delegation.token.AuthorizationGrant;
import edu.uiuc.ncsa.security.delegation.token.TokenForge;
import edu.uiuc.ncsa.security.oauth_1_0a.OAuthConstants;
import edu.uiuc.ncsa.security.oauth_1_0a.OAuthUtilities;
import edu.uiuc.ncsa.security.oauth_1_0a.client.OAClient;
import net.oauth.OAuthAccessor;
import net.oauth.OAuthMessage;

import java.net.URI;
import java.security.PublicKey;

import static edu.uiuc.ncsa.security.util.pkcs.KeyUtil.fromX509PEM;

/**
 * <p>Created by Jeff Gaynor<br>
 * on May 13, 2011 at  8:21:14 AM
 */
public class AuthorizationGrantIssuerImpl extends AuthorizationGrantIssuer {
    public AuthorizationGrantIssuerImpl(TokenForge tokenForge, URI address) {
        super(tokenForge, address);
    }

    @Override
    public AGResponse processAGRequest(AGRequest agReq) {
        try {
            OAuthMessage m = OAuthUtilities.getMessage(agReq.getServletRequest());
            if (!(agReq.getClient() instanceof OAClient)) {
                throw new NotImplementedException("This is only implemented for OAuth client. The class found was " + agReq.getClient().getClass().getName());
            }
            OAClient oaClient = (OAClient) agReq.getClient();

            OAuthAccessor accessor = OAuthUtilities.createOAuthAccessor(this, oaClient);
            if (oaClient.getSignatureMethod().equals(OAuthConstants.RSA_SHA1)) {
                // then there *has* to be a public key with the client. Use it.
                PublicKey pk = fromX509PEM(oaClient.getSecret());
                accessor.consumer.setProperty(OAuthConstants.PUBLIC_KEY, pk);
                accessor.setProperty(OAuthConstants.PUBLIC_KEY, pk);
            }
            // Now that we have the message and accessor, validate it or refuse to do anything.
            OAuthUtilities.validate(m, accessor);
            AuthorizationGrant ag = tokenForge.getAuthorizationGrant();
            AGResponseImpl agResponse = new AGResponseImpl();
            agResponse.setGrant(ag);
            // convert the list to a map.

            agResponse.setParameters(OAuthUtilities.getParameters(m));
            return agResponse;
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            }
            throw new GeneralException(e);
        }
    }
}
